@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.58 2000/08/03 23:31:37 nisse Exp $
+ * $Id: channel.h,v 1.59 2000/10/06 23:45:04 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -339,7 +339,7 @@ struct lsh_string *format_channel_eof(struct ssh_channel *channel);
 void channel_close(struct ssh_channel *channel);
 void channel_eof(struct ssh_channel *channel);
 
-struct close_callback *
+struct lsh_callback *
 make_channel_read_close_callback(struct ssh_channel *channel);
 
 struct exception_handler *
