@@ -1,6 +1,6 @@
 /* channel_commands.h
  *
- * $Id: channel_commands.h,v 1.14 2000/05/06 11:17:29 nisse Exp $
+ * $Id: channel_commands.h,v 1.15 2000/08/03 23:33:25 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -106,7 +106,7 @@ make_install_global_request_handler(UINT32 name,
      (name install_info)
      (super collect_info_2)
      (vars
-       (name . UINT32)))
+       (name . int)))
 */
 
 struct lsh_object *
