@@ -1,6 +1,6 @@
 /* dsa.c
  *
- * $Id: dsa.c,v 1.20 2000/09/19 20:50:42 nisse Exp $
+ * $Id: dsa.c,v 1.22 2000/10/06 11:11:05 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -239,13 +239,12 @@ do_dsa_sign_spki(struct signer *c,
 struct sexp *
 make_dsa_public_key(struct dsa_public *dsa)
 {
-  return sexp_l(2, SA(PUBLIC_KEY),
-		sexp_l(5, SA(DSA),
-		       sexp_l(2, SA(P), sexp_un(dsa->p), -1),
-		       sexp_l(2, SA(Q), sexp_un(dsa->q), -1),
-		       sexp_l(2, SA(G), sexp_un(dsa->g), -1),
-		       sexp_l(2, SA(Y), sexp_un(dsa->y), -1),
-		       -1), -1);
+  return sexp_l(5, SA(DSA),
+		sexp_l(2, SA(P), sexp_un(dsa->p), -1),
+		sexp_l(2, SA(Q), sexp_un(dsa->q), -1),
+		sexp_l(2, SA(G), sexp_un(dsa->g), -1),
+		sexp_l(2, SA(Y), sexp_un(dsa->y), -1),
+		-1);
 }
   
 static struct sexp *
@@ -423,7 +422,8 @@ do_dsa_verify(struct verifier *c, int algorithm,
 
 	break;
       }
-      
+
+#if DATAFELLOWS_WORKAROUNDS
     case ATOM_SSH_DSS_KLUDGE_LOCAL:
       {
 	UINT32 buf_length;
@@ -436,10 +436,11 @@ do_dsa_verify(struct verifier *c, int algorithm,
 
 	buf_length = signature_length / 2;
 
-	bignum_parse_u(s, buf_length, signature_data + buf_length);
+	bignum_parse_u(r, buf_length, signature_data);
 	bignum_parse_u(s, buf_length, signature_data + buf_length);
 	break;
       }
+#endif
     case ATOM_SPKI:
       {
 	struct simple_buffer buffer;
