@@ -2,7 +2,7 @@
  *
  * Simple mark&sweep garbage collector.
  *
- * $Id: gc.h,v 1.5 1999/02/25 00:33:48 nisse Exp $ */
+ * $Id: gc.h,v 1.6 1999/09/07 20:07:55 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -28,10 +28,6 @@
 
 #include "lsh.h"
 
-#if 0
-void gc_register_global(struct lsh_object *o);
-#endif
-
 void gc_register(struct lsh_object *o);
 void gc_kill(struct lsh_object *o);
 
