@@ -2,7 +2,7 @@
  *
  * Decode ssh2 keys.
  *
- * $Id: lsh-decode-key.c,v 1.3 2000/09/03 21:47:38 nisse Exp $
+ * $Id: lsh-decode-key.c,v 1.4 2000/10/03 21:42:53 nisse Exp $
  */
 
 #include "dsa.h"
@@ -166,7 +166,8 @@ do_decode_key(struct abstract_write *s,
 		  return;
 		}
 
-	      COMMAND_RETURN(self->c, make_dsa_public_key(&dsa));
+	      COMMAND_RETURN(self->c, sexp_l(2, sexp_a(ATOM_PUBLIC_KEY),
+					     make_dsa_public_key(&dsa), -1));
 	      dsa_public_free(&dsa);
 	      break;
 	    }
