@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: randomness.c,v 1.17 1999/11/07 21:02:12 nisse Exp $ */
+ * $Id: randomness.c,v 1.18 2000/08/04 00:04:49 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -55,7 +55,7 @@
      (super randomness)
      (vars
        (hash object hash_instance)
-       (pos simple UINT32)
+       (pos . UINT32)
        (buffer space UINT8)))
 */
 
