@@ -2,7 +2,7 @@
  *
  * Handle child processes.
  *
- * $Id: reaper.c,v 1.12 1999/05/12 21:35:23 nisse Exp $
+ * $Id: reaper.c,v 1.13 2000/05/06 11:17:39 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -151,8 +151,8 @@ struct reap *make_reaper(void)
   return &closure->super;
 }
 
-/* FIXME: Perhaps this function should return a suitable exit code? */
-void reaper_run(struct reap *r, struct io_backend *b)
+void
+reaper_run(struct reap *r, struct io_backend *b)
 {
   CAST(reaper, self, r);
   
