@@ -1,6 +1,6 @@
 /* server_pty.c
  *
- * $Id: server_pty.c,v 1.16 2000/06/27 11:36:42 nisse Exp $
+ * $Id: server_pty.c,v 1.17 2000/07/27 21:45:42 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -222,9 +222,11 @@ close_master:
   unsigned int i, j;
   int saved_errno;
 
-  for (i = 0; i < sizeof(first); i++)
+#define CONST_STRLEN(s) (sizeof((s)) - sizeof(""))
+  
+  for (i = 0; i < CONST_STRLEN(first); i++)
     {
-      for (j = 0; j < sizeof(second); j++) 
+      for (j = 0; j < CONST_STRLEN(second); j++) 
         {
 	  snprintf(master, sizeof(master),
 		   PTY_BSD_SCHEME_MASTER, first[i], second[j]);
