@@ -2,7 +2,7 @@
  *
  * Declarations for Thomas Wu's Secure Remote Password Protocol
  *
- * $Id: srp.h,v 1.3 2000/05/31 17:27:04 nisse Exp $ */
+ * $Id: srp.h,v 1.4 2000/06/04 19:59:19 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -76,13 +76,15 @@ srp_process_reply_msg(struct dh_instance *dh, struct lsh_string *packet);
 
 struct lsh_string *
 srp_make_client_proof(struct dh_instance *dh,
+		      struct lsh_string **m2,
 		      mpz_t x);
 
 struct lsh_string *
 srp_process_client_proof(struct dh_instance *dh, struct lsh_string *packet);
 
 int
-srp_process_server_proof(struct dh_instance *dh, struct lsh_string *packet);
+srp_process_server_proof(struct lsh_string *m2,
+			 struct lsh_string *packet);
 
 struct dh_method *
 make_srp1(struct randomness *r);
