@@ -1,6 +1,6 @@
 /* tcpforward.c
  *
- * $Id: tcpforward.c,v 1.53 2000/07/27 21:48:41 nisse Exp $
+ * $Id: tcpforward.c,v 1.54 2000/10/07 00:00:30 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -551,7 +551,7 @@ do_tcpip_cancel_forward(struct global_request *s UNUSED,
 	  assert(port->socket);
 	  verbose("Cancelling a requested tcpip-forward.\n");
 
-	  close_fd(port->socket, 0);
+	  close_fd(port->socket);
 	  port->socket = NULL;
 
 	  COMMAND_RETURN(c, NULL);
