@@ -1,6 +1,6 @@
 /* tty.h
  *
- * $Id: tty.h,v 1.8 2000/02/02 22:44:45 nisse Exp $ */
+ * $Id: tty.h,v 1.9 2000/04/23 17:48:54 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -35,8 +35,10 @@ int tty_makeraw(int fd);
 int tty_getwinsize(int fd, UINT32 *w, UINT32 *h, UINT32 *wp, UINT32 *hp);
 int tty_setwinsize(int fd, UINT32 w, UINT32 h, UINT32 wp, UINT32 hp);
 
-struct lsh_string *tty_encode_term_mode(struct termios *ios);
-int tty_decode_term_mode(struct termios *ios, UINT32 t_len, UINT8 *t_modes);
+struct lsh_string *
+tty_encode_term_mode(struct termios *ios);
+int
+tty_decode_term_mode(struct termios *ios, UINT32 t_len, const UINT8 *t_modes);
 
 #if HAVE_CFMAKERAW
 #define CFMAKERAW cfmakeraw
